# Copyright 2021 The SQLNet Company GmbH

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

"""
This module helps you handle your current project.
"""

from sys import modules
from types import ModuleType

from getml.engine import is_alive

from .attrs import __getattr__, _all
from .containers import DataFrames, Hyperopts, Pipelines


def __dir__():
    return _all


__all__ = _all + ["DataFrames", "Hyperopts", "Pipelines"] + ["__getattr__"]


class ProjectModule(ModuleType):
    def __repr__(self):
        output = ""
        # resolving name has the side effect of reaching out to the monitor and
        # the engine and therefore triggers a message if either is unreachable
        project_name = self.name  # pylint: disable=E1101
        if is_alive():
            output += f"Current project:\n\n{project_name}"
        return output


modules[__name__].__class__ = ProjectModule
